package org.netkernel.cp;

import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSNodeListImpl;
import org.netkernel.layer0.util.MultiMap;

import java.util.*;

public class ControlPanelConfig
{
	private List<IHDSNode> mConfigs;
	private IHDSNode mPanelDoc;
	private Map<String, IHDSNode> mPanelMap;
	private Map<String, IHDSNode> mPathToTool;
	
	public ControlPanelConfig()
	{	mConfigs = new ArrayList<IHDSNode>(32);
	}
	
	public void addConfig(IHDSNode aConfig)
	{
		//validate
		//TODO validate control panel config
		
		mConfigs.add(aConfig);
	}
	
	public void close()
	{		
		MultiMap icons = new MultiMap(16,32);
		MultiMap widgets = new MultiMap(16,4);
		mPathToTool = new HashMap<String, IHDSNode>(128);
		
		//index all icons and widgets
		for (IHDSNode configRoot : mConfigs)
		{	for (IHDSNode configNode : configRoot.getNodes("//controlpanel/*"))
			{	String type=configNode.getName();
				if (type.equals("icon"))
				{	String panelId=configNode.getFirstValue("panel").toString().trim();
					icons.put(panelId, configNode);
					String path=configNode.getFirstValue("path").toString().trim();
					mPathToTool.put(path,configNode);
				}
				else if (type.equals("widget"))
				{	String panelId=configNode.getFirstValue("panel").toString().trim();
					widgets.put(panelId, configNode);
				}
			}
		}
		
		//build proxy icons on extras for panels with order>6
		final float MAX_ORDER=6.0f;
		final String EXTRAS_PANEL_ID="urn:org:netkernel:nkse:control:panel:extra";
		for (IHDSNode configRoot : mConfigs)
		{	for (IHDSNode configNode : configRoot.getNodes("//controlpanel/*"))
			{	String type=configNode.getName();
				if (type.equals("panel"))
				{	String panelId=configNode.getFirstValue("id").toString().trim();
					String title=configNode.getFirstValue("title").toString().trim();
					String subtitle=configNode.getFirstValue("subtitle").toString().trim();
					String icon=(String)configNode.getFirstValue("icon");
					String orderString=(String)configNode.getFirstValue("order");
					float order=orderString==null?Float.MAX_VALUE:Float.parseFloat(orderString);
					if (order>=MAX_ORDER)
					{	HDSBuilder b = new HDSBuilder();
						b.pushNode("icon");
						b.addNode("title",title);
						b.addNode("subtitle",subtitle);
						String path="/panel/"+panelId;
						b.addNode("path",path);
						if (icon!=null)
						{	b.addNode("icon",icon);
						}
						b.addNode("panel",EXTRAS_PANEL_ID);
						b.addNode("order",orderString);
						b.popNode();
						IHDSNode iconConfig=b.getRoot().getFirstNode("/*");
						icons.put(EXTRAS_PANEL_ID, iconConfig);
						//mPathToTool.put(path,iconConfig);
					}
				}
			}
		}
		
		//build documents
		HDSBuilder panelsBuilder = new HDSBuilder();
		mPanelMap = new HashMap<String,IHDSNode>(16);
		
		panelsBuilder.pushNode("panels");
		for (IHDSNode configRoot : mConfigs)
		{	for (IHDSNode configNode : configRoot.getNodes("//controlpanel/*"))
			{	String type=configNode.getName();
				if (type.equals("panel"))
				{	String panelId=configNode.getFirstValue("id").toString().trim();
					panelsBuilder.pushNode("panel");
					panelsBuilder.importChildren(configNode);
					panelsBuilder.popNode();
					
					HDSBuilder b = new HDSBuilder();
					b.pushNode("panel");
					b.importChildren(configNode);
					b.pushNode("icons");
					
					IHDSNodeList iconList = new HDSNodeListImpl((List<IHDSNode>)icons.get(panelId));
					Comparator<IHDSNode> cp = new Comparator<IHDSNode>()
					{	public int compare(IHDSNode o1, IHDSNode o2)
						{	String id1=(String)o1.getFirstValue("order");
							String id2=(String)o2.getFirstValue("order");
							int int1, int2;
							if (id1==null)
							{	int1=Integer.valueOf(Integer.MAX_VALUE);
							}
							else
							{	int1=Integer.parseInt(id1);
							}
							if (id2==null)
							{	int2=Integer.valueOf(Integer.MAX_VALUE);
							}
							else
							{	int2 = Integer.parseInt(id2);
							}
							int v=int1-int2;
							return v;
						}
					};
					
					iconList=iconList.sort(cp);
					for (IHDSNode iconNode : iconList)
					{	String orderString=(String)iconNode.getFirstValue("order");
						if (orderString==null || Integer.parseInt(orderString)>=0)
						{	// hide icons with an order < 0
							b.pushNode("icon");
							b.importChildren(iconNode);
							b.popNode();
						}
					}
					b.popNode();
					b.pushNode("widgets");
					for (IHDSNode iconNode : (List<IHDSNode>)widgets.get(panelId))
					{	b.pushNode("widget");
						b.importChildren(iconNode);
						b.popNode();
					}
					b.popNode();
					
					mPanelMap.put(panelId,b.getRoot());
				}
			}
		}
		
		
		
		mPanelDoc = panelsBuilder.getRoot();
		mConfigs=null;
	}
	
	public IHDSNode getPanels()
	{	return mPanelDoc;
	}
	
	public IHDSNode getPanel(String aId)
	{	return mPanelMap.get(aId);
	}
	
	public IHDSNode getIconForPath(String aPath)
	{	IHDSNode result=mPathToTool.get(aPath);
		if (result==null)
		{	//attempt search of starting path
			int pathLength=0;
			for (String path: mPathToTool.keySet())
			{	if (aPath.startsWith(path) && path.length()>pathLength)
				{	result=mPathToTool.get(path);
					pathLength=path.length();
				}
			}
		}
		return result;
	}
}
